void mpris_fullscreen_change(void);
void mpris_volume_change(void);
void mpris_loop_status_change(void);
void mpris_playback_status_change(void);
void mpris_position_change(double pos);
void mpris_metadata_change(void);
void mpris_init(Evas_Object *win);
void mpris_shutdown(void);
