#ifndef _WINLIST_H__
#define _WINLIST_H__ 1

void win_list_show(Evas_Object *win);
void win_list_hide(Evas_Object *win);
void win_list_toggle(Evas_Object *win);
void win_list_sel_update(Evas_Object *win);
void win_list_size_update(Evas_Object *win);
void win_list_content_update(Evas_Object *win);

#endif
